const fs = require('fs')
const fsExtra = require('fs-extra')
const { join, resolve } = require('path')
const ciqlJSON = require("ciql-json")
const { cwd } = require('process')
const { writeCryptJson, readCryptJson, cryptG } = require('../../libs/function')
const webpack = require('webpack')
const nodeExternals = require('webpack-node-externals')

function build(path) {
  return new Promise(async (resolveOK, reject) => {
    try {
      const { name: apiname } = ciqlJSON.create(readCryptJson(join(cwd(), ".ciql/data/.info")))
        .extract('{name}')
        .getData()

      fs.mkdirSync(path, { recursive: true });

      webpack({
        entry: resolve('index.js'),
        mode: 'production',
        target: 'node', //ignore built-in modules like path, fs, etc. 
        externals: [nodeExternals()], //ignore all modules in node_modules folder 
        output: {
          path: resolve(path),
          filename: 'index.js',
        },
      }, (err, stats) => {
        if (err || stats.hasErrors()) {
          reject(err || 'Errors in stats');
        } else {

          let file = fs.readFileSync(join(path, "index.js")).toString()
          let cfile = cryptG(file, join(cwd(), '.ciql', 'keys'))

          fsExtra.copySync("./.ciql", join(path, ".ciql"))

          ciqlJSON
            .create({})
            .set("date", new Date())
            .set("code", cfile)
            .save(join(path, '.ciql', 'data', 'releases.json'))


          writeCryptJson(
            ciqlJSON
              .create(readCryptJson(join(path, ".ciql/data/.info")))
              .set("node_env", "production")
              .set("status", "built")
              .getData(),
            join(path, ".ciql/data/.info")
          )

          writeCryptJson({}, join(path, ".ciql/data/.history"))
          writeCryptJson({}, join(path, ".ciql/data/.servers"))
          writeCryptJson({}, join(path, ".ciql/data/.databases"))

          fs.writeFileSync(join(path, ".env"), `PORT=3983\nAPI_NAME=${apiname}\nNODE_ENV=production`)
          fs.writeFileSync(resolve(path, "index.js"), `module.exports = require("ciql-secry").server()`)

          fs.copyFileSync(join(__dirname, "../../templates/READMEProd"), join(path, "README.md"))
          fsExtra.copySync(join(__dirname, "../../templates/public/"), join(path, "public"))
          fsExtra.copySync(join(cwd(), "package.json"), join(path, "package.json"))

          resolveOK()
        }
      })

    } catch (error) {
      console.log(error);
      reject(error.message)
    }
  })
}

module.exports = build
