const file = require('../../libs/files.js')
const { createFolder } = require('../../libs/utils');
const { join } = require('path')
const { existsSync } = require('fs')
const CiqlJSON = require('ciql-json');
const { readCryptJson, writeCryptJson } = require('../../libs/function.js');
const { cwd } = require('process');

function generate(fileSchema = "schema.js") {
  return new Promise(async (resolve, reject) => {
    try {
      if (!existsSync(fileSchema)) {
        reject("Inexist File " + fileSchema);
      } else {
        const schema = require(join(cwd(), './' + fileSchema))
        const { name: apiname } = CiqlJSON
          .create(readCryptJson(join(cwd(), ".ciql/data/.info")))
          .extract("{name}")
          .getData()

        let allTables = Object.keys(schema)
        let tables = allTables.filter(t => !/^\_/.test(t))

        createFolder('api')
        createFolder('docs')
        createFolder('public')
        createFolder('server')
        createFolder('db')

        //SERVER
        file.writeApp(apiname, tables)
        file.writeIndex(apiname)
        file.writeGbRoute(apiname, tables)

        //DB
        file.writeIndexDB(schema, allTables)
        file.writeRelations(schema, allTables)

        //DOCS
        file.writeDocsIndex(tables)
        file.writeDocsInfo(apiname)

        //PUBLIC
        file.writeIco()

        //INDEX
        file.writeIndexRoot()

        //API
        tables.forEach(async (table, i) => {
          createFolder(join("api", table))
          file.writeRoute(table)
          file.writeTest(table)
          file.writeApiDocs(schema,table)
        });

        //CIQL
        writeCryptJson(
          CiqlJSON
            .create(readCryptJson(join(cwd(), ".ciql/data/.info")))
            .set("status", "generated")
            .getData(),
          join(cwd(), ".ciql/data/.info")
        )


        resolve("Files genrated")
      }
    } catch (error) {
      reject(error.message)
    }
  })

}

module.exports = generate
