const { resolve, basename, join } = require('path')
const { NodeSSH } = require('node-ssh')
const { logInfo, logError } = require("../../libs/utils")
const { writeCryptJson, readCryptJson } = require('../../libs/function')
const build = require('./build')
const CiqlJson = require("ciql-json")
const ssh = new NodeSSH()
const fs = require('fs');
const { cwd } = require('process')


function publish({ servername, host, port, user, pass }, { db, dbConn, dbId }, remotePath, cmd) {
    return new Promise(async (resolveOK, reject) => {
        try {
            logInfo("Connecting to server ", servername)
            ssh.connect({
                host: host,
                username: user,
                password: pass,
                port: port
            }).then(async () => {


                logInfo("Build and compress project")

                // let pathTemp = join(os.tmpdir(), "build", getName())
                let pathTemp = resolve(cwd(), "tmp/build")
                await build(pathTemp)
                let localPath = pathTemp

                logInfo("Connection to " + db + "database")

                writeCryptJson(dbConn, join(resolve(localPath), ".ciql/keys/.db"))
                writeCryptJson(CiqlJson
                    .create(readCryptJson(join(resolve(localPath), ".ciql/data/.info")))
                    .set("db", { connected: true, id: dbId })
                    .getData(),
                    join(resolve(localPath), ".ciql/data/.info")
                )


                logInfo("Deploy project to server")

                ssh.putDirectory(resolve(localPath), resolve(remotePath),
                    {
                        recursive: true,
                        concurrency: 10,
                        validate: (itemPath) => {
                            const baseNameFolder = basename(itemPath)
                            return baseNameFolder.substr(0, 1) === '.' || // allow dot (hidden) files
                                baseNameFolder !== 'node_modules' // do not allow node_modules
                        }
                    })
                    .then(() => {
                        fs.rmSync(resolve(pathTemp), { recursive: true }, (err) => { if (err) { throw err } })
                        logInfo("execute '" + cmd + "'")
                        ssh.execCommand(cmd, { cwd: resolve(remotePath), stream: 'stdout', options: { pty: true } })
                            .then(function (result) {
                                console.log(result.stdout);
                                console.log(result.stderr);
                                resolveOK("Application deployed to " + servername + " server")
                            })
                            .catch(error => {
                                logError(error);
                                reject(error)
                            })
                    })
                    .catch(error => {
                        console.error(error);
                        reject(error)
                    })


            }).catch(error => {
                console.error(error);
                reject(error)
            })

        } catch (error) {
            console.error(error);
            reject(error)
        }
    })
}

module.exports = publish
