const file = require('../../libs/files.js')
const { createFolder } = require('../../libs/utils')

const { join } = require('path')
const { existsSync, rm, readdir } = require('fs')
const CiqlJSON = require('ciql-json');
const { cwd } = require('process');
const { writeCryptJson, readCryptJson } = require('../../libs/function.js');

async function update(fileSchema = "schema.js") {
  return new Promise(async (resolve, reject) => {
    try {
      if (!existsSync(fileSchema)) {
        reject("Inexist File " + fileSchema);
      } else {

        const schema = require(join(cwd(), './' + fileSchema))
        const { name: apiname } = CiqlJSON
          .create(readCryptJson(join(cwd(), ".ciql/data/.info")))
          .extract("{name}")
          .getData()

        let allTables = Object.keys(schema)
        let tables = allTables.filter(t => !/^_/.test(t))

        file.writeGbRoute(apiname, tables)

        //
        file.writeIndexDB(schema, allTables)
        file.writeRelations(schema, allTables)

        file.writeDocsIndex(tables)

        readdir('./api/', (err, files) => {
          if (err) throw err
          files.forEach(f => {
            if (!tables.includes(f)) {
              rm(join("api", f), { recursive: true }, (error) => {
                if (error) { throw error }
              })
            }
          })
        })

        tables.forEach(async (table, i) => {
          if (!existsSync(join("api", table))) {
            createFolder(join("api", table))
            file.writeTest(table)
            file.writeRoute(table)
            file.writeApiDocs(schema, table)
          }
        });
      }
      writeCryptJson(
        CiqlJSON
          .create(readCryptJson(join(cwd(), ".ciql/data/.info")))
          .set("updatedAt", new Date())
          .getData(), join(cwd(), ".ciql/data/.info"))

      resolve("Files updated")
    } catch (error) {
      reject(error.message)
    }
  })
}

module.exports = update
