const ciqlJSON = require("ciql-json")
const { join } = require("path")
const table = require("tty-table")
const { readCryptJson } = require("../../libs/function")
const { cwd } = require("process")


function list() {
  return new Promise(async (resolveOK, reject) => {
    try {

      const header = [
        { value: "Config name", color: "red",  },
        { value: "Dialect" },
        { value: "Host" },
        { value: "Port" },
        { value: "User" },
        { value: "Database", color: "green" }
      ]

      const rows = []
      const options = {
        borderStyle: "solid",
        borderColor: "blue",
        headerAlign: "center",
        headerColor: "cyan",
        align: "left",
        color: "white",
        width: "100%"
      }
      const {db : {id}} = ciqlJSON
        .create(readCryptJson(join(cwd(), ".ciql/data/.info")))
        .extract("{db}")
        .getData()

      const infos = ciqlJSON.create(readCryptJson(join(cwd(), ".ciql/data/.databases"))).getData()
      const databases = ciqlJSON.create(readCryptJson(join(cwd(), ".ciql/data/.databases"))).getKeys()

      databases.forEach(d => {
        if (d !== "iat") {
          rows.push([
            infos[d]["id"] === id ? "[*] " + d : d,
            infos[d]["dialect"],
            infos[d]["host"],
            infos[d]["port"],
            infos[d]["user"],
            infos[d]["database"],
          ])
        }
      });
      console.log("\nDatabases configuration list");
      console.log(table(header, rows, options).render());

      resolveOK("OK")
    } catch (error) {
      console.log(error);
      reject(error.message)
    }
  })
}

module.exports = list
