const ciqlJSON = require("ciql-json")
const { join } = require("path")
const table = require("tty-table")
const { readCryptJson } = require("../../libs/function")
const moment = require("moment")

function inspect() {
  return new Promise(async (resolveOK, reject) => {
    try {

      const header = [
        { value: "Keys", color: "red" },
        {
          value: "Values", width: 30,
          formatter: function (value) {
            if (typeof (value) === "object") {
              let string = ""
              let keys = Object.keys(value)
              keys.forEach(elt => {
                string += `${elt} : ${value[elt]} \n`
              });
              return string
            } else {
              return value
            }
          }
        },
      ]

      const rows = []
      const options = {
        borderStyle: "solid",
        borderColor: "blue",
        headerAlign: "center",
        headerColor: "cyan",
        align: "left",
        color: "white",
        width: "100%"
      }

      const jsonFile = readCryptJson(join('.ciql', 'data', '.info'))
      const infos = ciqlJSON.create(jsonFile).getData()
      const keys = ciqlJSON.create(jsonFile).getKeys()


      keys.forEach(k => {
        if (!["id", "iat", "db", "secure", "lastVersion", "node_env"].includes(k)) {
          rows.push([
            k,
            ["createdAt", "updatedAt"].includes(k) ?  moment(infos[k]).format("ddd DD/MM/YYYY HH:mm:ss") : infos[k]
          ])
        }
      });

      console.log("\nProject Basic information");
      console.log(table(header, rows, options).render());



      resolveOK("OK")
    } catch (error) {
      reject(error.message)
    }
  })
}

module.exports = inspect
