const { createFolder } = require('../../libs/utils.js')
const file = require('../../libs/files.js')
const { join, resolve: resolveP } = require('path')
const chalk = require('chalk');
const log = console.log
const { exec } = require('child_process')
const { prompt } = require('enquirer');


function init(apiname, description, author) {
  return new Promise(async (resolve, reject) => {
    try {

      //créer les dossiers de l'api et d'identification
      createFolder(apiname)
      createFolder(join(apiname, '.ciql'))
      createFolder(join(apiname, '.ciql', 'keys'))
      createFolder(join(apiname, '.ciql', 'data'))

      //créer les fichiers de demarrage
      file.writeSchema(apiname)
      file.writePackage(apiname, description, author)
      file.writeReleases(apiname)
      file.writePass(apiname, description, author)
      file.writeReadme(apiname)
      file.writeGitIgnore(apiname)
      file.writeEnv(apiname)

      //installer les packages
      exec('yarn --help', { cwd: resolveP(apiname) }, async (err, stdout, stderr) => {
        if (err) {

          let npm = exec('npm i', { cwd: resolveP(apiname) })
          console.log("npm install");
          npm.stdout.on('data', (data) => {
            console.log(data);
          })
          npm.stderr.on('data', (data) => {
            console.log(data);
          })

        } else {
          let { yarn } = await prompt([
            {
              type: 'confirm',
              name: 'yarn',
              initial : "yes",
              message: 'yarn detected. Do you want to use it ?',
            },])


          if (yarn) {

            let yarn = exec('yarn install', { cwd: resolveP(apiname) })
            yarn.stdout.on('data', (data) => {
              console.log(data);
            })
            yarn.stderr.on('data', (data) => {
              console.log(data);
            })

          } else {

            let npm = exec('npm i', { cwd: resolveP(apiname) })
            console.log("npm install");
            npm.stdout.on('data', (data) => {
              console.log(data);
            })
            npm.stderr.on('data', (data) => {
              console.log(data);
            })

          }

        }
      })

      //afficher le message de fin
      log();
      log(chalk.green.bold('Finish'), ' : ', 'API created in ', chalk.yellow.bold(resolveP(apiname)))
      log(chalk.green.bold('Instructions') + ' : Go in ', chalk.green.bold(apiname), ", And follow instructions in", chalk.blue.bold("README"), "file")
      resolve("ok")
      log();
    } catch (error) {
      reject(error.message)
    }
  })
}

module.exports = init
