const ciqlJSON = require("ciql-json")
const { join } = require("path")
const table = require("tty-table")
const { readCryptJson } = require("../../libs/function")
const { cwd } = require("process")
const moment = require('moment')


function list() {
  return new Promise(async (resolveOK, reject) => {
    try {
      const header = [
        { value: "Id", color: "red", },
        { value: "Task", align: "left" },
        { value: "Created At" },
        { value: "End" },
        { value: "Time" },
        {
          value: "Ended?",
          formatter: function (value) {
            if (value === true) {
              value = this.style("yes", "bgGreen", "black")
            } else {
              value = this.style("no", "bgRed", "black")
            }
            return value
          }
        }
      ]

      const rows = []
      const options = {
        borderStyle: "solid",
        borderColor: "blue",
        headerAlign: "center",
        headerColor: "cyan",
        align: "center",
        color: "white",
        width: "100%"
      }

      const infos = ciqlJSON.create(readCryptJson(join(cwd(), ".ciql/data/.todos"))).getData()
      const todos = ciqlJSON.create(readCryptJson(join(cwd(), ".ciql/data/.todos"))).getKeys()

      todos.forEach(d => {
        if (d !== "iat") {
          rows.push([
            d,
            infos[d]["task"],
            moment(infos[d]["start"]).format("ddd DD/MM/YYYY HH:mm"),
            infos[d]["end"] && moment(infos[d]["end"]).format("ddd DD/MM/YYYY HH:mm"),
            moment(infos[d]["start"]).from(infos[d]["end"]).replace("ago",""),
            infos[d]["checked"],
          ])
        }
      });
      console.log("\nTodos list");
      console.log(table(header, rows, options).render());
      console.log(table([
        ["Total task", rows.length],
        ["Ended task", todos.filter(elt => infos[elt].checked).length],
        ["Percentage", (todos.filter(elt => infos[elt].checked).length/rows.length * 100).toFixed(2) + "%"],
      ]).render());

      resolveOK("OK")
    } catch (error) {
      reject(error.message)
    }
  })
}

module.exports = list
