const { logError, logInfo } = require('../libs/utils');
const { existsSync } = require('fs');
const { crypt, dcrypt, algo } = require('./utils');
const fs = require("fs");
const { join } = require('path');
const ciqlJSON = require("ciql-json")
const jwt = require("jsonwebtoken")
const { cwd } = require('process');
const md5 = require('md5')

exports.validity = () => {
    try {
        let dataKeys = join(cwd(), ".ciql", "keys")
        let key = fs.readFileSync(join(dataKeys, ".pass")).toString()
        const { id } = ciqlJSON
            .create(this.readCryptJson(".ciql/data/.info"))
            .extract("{id}")
            .getData()

        if (md5(id) === key) {
            return true
        } else {
            logError("This project is invalid, the keys are corrupted");
            return false
        }
    } catch (error) {
        logError(error.message)
    }


}

exports.checkLock = () => {
    const { secure: { active } } = ciqlJSON
        .create(this.readCryptJson(join(cwd(), '.ciql/data/.info')))
        .extract("{ secure : { active } }")
        .getData()

    if (active === false) {
        return true
    } else {
        logError("This project is locked");
        logInfo("type ciql secure unlock to allow ciql command");
        return false
    }
}

exports.verify = (callback) => {
    if (existsSync(join(cwd(), '.ciql', 'keys/.pass'))) {
        if (this.validity()) {
            callback()
        }
    }
    else
        logError("This is not ciql project root");
}

exports.cryptG = (value, folder) => {
    if (value !== null) {
        if (fs.existsSync(join(folder, '.passiv.key')) && fs.existsSync(join(folder, '.iv.key'))) {
            let passiv = fs.readFileSync(join(folder, '.passiv.key')).toString()
            let iv = fs.readFileSync(join(folder, '.iv.key')).toString()
            let Bpassiv = Buffer.from(passiv, 'base64')
            let Biv = Buffer.from(iv, 'base64')
            return crypt(value, algo, Bpassiv, Biv)
        } else {
            console.log('keys files not exist');
            return null
        }
    } else {
        return null
    }
}
exports.dcryptG = (value, folder) => {
    if (value !== null) {
        if (fs.existsSync(join(folder, '.passiv.key')) && fs.existsSync(join(folder, '.iv.key'))) {
            let passiv = fs.readFileSync(join(folder, '.passiv.key')).toString()
            let iv = fs.readFileSync(join(folder, '.iv.key')).toString()
            let Bpassiv = Buffer.from(passiv, 'base64')
            let Biv = Buffer.from(iv, 'base64')
            return dcrypt(value, algo, Bpassiv, Biv)
        } else {
            console.log('keys files not exist');
            return null
        }
    } else {
        return null
    }
}

exports.readCryptJson = (source, target, apiname) => {
    let dataKeys = join(apiname ? join(cwd(), apiname) : cwd(), ".ciql", "keys")
    let key = fs.readFileSync(join(dataKeys, ".pass")).toString()
    let dataCrypt = fs.readFileSync(source).toString()
    let dataDcrypt = this.dcryptG(dataCrypt, dataKeys)
    let dataJWT = jwt.verify(dataDcrypt, key)

    if (target !== undefined) {
        fs.writeFileSync(target, dataJWT)
    } else {
        return dataJWT
    }
}

exports.writeCryptJson = (data, file, apiname) => {
    let dataKeys = join(apiname ? join(cwd(), apiname) : cwd(), ".ciql", "keys")
    let key = fs.readFileSync(join(dataKeys, ".pass")).toString()
    let dataJWT = jwt.sign(data, key)
    fs.writeFileSync(file, this.cryptG(dataJWT, dataKeys))
}

exports.hasReleases = () => {
    const release = ciqlJSON
        .open(join(".ciql", "data", "releases.json"))
        .getKeys()

    if (release.length === 0) {
        logError("Not release saved")
        logInfo("Type 'ciql release' to save release")
        return false
    } else {
        return true
    }
}


const getStatus = () => {
    const { status } = ciqlJSON
        .create(this.readCryptJson(".ciql/data/.info"))
        .extract("{status}")
        .getData()

    return status
}

exports.getName = () => {
    const { name } = ciqlJSON
        .create(this.readCryptJson(".ciql/data/.info"))
        .extract("{name}")
        .getData()

    return name
}

exports.checkStatus = (...acceptedStatus) => {
    const status = getStatus()
    if (acceptedStatus.includes(status)) {
        return true
    } else {
        logError("Can't do this action in", status, "project")
        return false
    }
}

exports.checkGit = () => {
    if (fs.existsSync(join(cwd(), '.git'))) {
        return true
    } else {
        logError("Neither this nor any of its parent directories is a git repository")
        logInfo("Type git init to initialyse git repository")
        return false
    }
}

exports.checkVersion = (version) => {
    return /^[\d]+\.[\d]+\.[\d]+$/.test(version)
}

exports.verifyVersion = (v1, v2) => {
    if (this.checkVersion(v1) && this.checkVersion(v2)) {
        v1 = v1.split(".").map(v => parseInt(v))
        v2 = v2.split(".").map(v => parseInt(v))

        if (
            (v1[0] > v2[0]) ||
            (v1[0] === v2[0] && v1[1] > v2[1]) ||
            (v1[0] === v2[0] && v1[1] === v2[1] && v1[2] > v2[2])
        ) {
            return true
        }
        else {
            logError("New version must be great than old version")
            return false
        }
    } else {
        logError("Invalid version")
        return false
    }
}

exports.getStatus = getStatus